//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: IndexLookupCommand.cpp,v 1.2 2004/09/06 14:01:14 jsnelson Exp $
//

#include "IndexLookupCommand.hpp"

using namespace DbXml;
using namespace std;

string IndexLookupCommand::getCommandName() const
{
	return "indexlookup";
}

string IndexLookupCommand::getBriefHelp() const
{
	return "Looks up an index on the default container";
}

string IndexLookupCommand::getMoreHelp() const
{
	return
		string("Takes arguments of the index descriptor string, the namespace and\n") +
		string("name of the element or attribute, and optionally the parent namespace\n") +
		string("and name, and the value to lookup . This command uses the\n") +
		string("XmlContainer::lookupIndex() method.\n");
}

void IndexLookupCommand::execute(Args &args, Environment &env)
{
	if(args.size() < 4 || args.size() > 7 || args.size() == 5) {
		throw CommandException("Wrong number of arguments");
	}
	env.testContainer();
	env.deleteResults();

	XmlValue value;
	if(args.size() == 7) {
		value = args[6];
	}

	if(env.txn()) {
		if(args.size() == 4 || args[5] == "") {
			env.results() = new XmlResults(env.container()->
						       lookupIndex(*env.txn(),
								   env.context(),
								   args[2], args[3],
								   args[1], value,
								   DBXML_LAZY_DOCS));
		}
		else {
			env.results() = new XmlResults(env.container()->
						       lookupIndex(*env.txn(),
								   env.context(),
								   args[2], args[3],
								   args[4], args[5],
								   args[1], value,
								   DBXML_LAZY_DOCS));
		}
	}
	else {
		if(args.size() == 4 || args[5] == "") {
			env.results() = new XmlResults(env.container()->
						       lookupIndex(env.context(),
								   args[2], args[3],
								   args[1], value,
								   DBXML_LAZY_DOCS));
		}
		else {
			env.results() = new XmlResults(env.container()->
						       lookupIndex(env.context(),
								   args[2], args[3],
								   args[4], args[5],
								   args[1], value,
								   DBXML_LAZY_DOCS));
		}
	}

	if(env.context().getEvaluationType() == XmlQueryContext::Eager) {
		if(env.verbose()) cout << env.results()->size() << " objects returned for eager index lookup '"
				       << args[1] << "'" << endl << endl;
	}
	else {
		if(env.verbose()) cout << "Lazy index lookup '" << args[1] << "' completed" << endl << endl;
	}
}

