//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: InsertAfterCommand.cpp,v 1.4 2004/10/18 22:52:13 gmf Exp $
//

#include "InsertAfterCommand.hpp"

using namespace DbXml;
using namespace std;

string InsertAfterCommand::getCommandName() const
{
  return "insertafter";
}

string InsertAfterCommand::getBriefHelp() const
{
  return "Insert after nodes from the documents in the last results returned";
}

string InsertAfterCommand::getMoreHelp() const
{
  return
    string("Takes the path to the nodes to insert after, the object to insert,\n") +
    string("the name of the object, and the object's content as arguments.\n") +
    string("Valid objects are: element, attribute, text, processinginstruction,\n") +
    string("and comment. The results of the modifications are left in the\n") +
    string("default results. This command creates an XmlModify object using\n") +
    string("XmlManager::createModify(), then adds an insert step to it using\n") +
    string("XmlModify::addInsertAfterStep(). It then executes using\n") +
    string("XmlModify::execute().");
}

void InsertAfterCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 5) {
    throw CommandException("Wrong number of arguments");
  }
  env.testResults();

  XmlModify::XmlObject o = getObject(args[2]);
  if(o == (XmlModify::XmlObject)-1) {
    throw CommandException("Unknown object type: " + args[2]);
  }

  if(env.verbose()) cout << "Inserting after nodes: " << args[1] << " an object of type: " << args[2]
                         << " with name: " << args[3] << " and content: " << args[4] << endl;

  XmlModify modify = env.db().createModify();
  env.results()->reset();

  unsigned int modifications;
  if(env.txn()) {
	  XmlTransaction myTxn = env.childTransaction();
	  modify.addInsertAfterStep(env.db().prepare(myTxn, args[1], env.context()), o, args[3], args[4]);
	  modifications = modify.execute(myTxn, *env.results(), env.context(), env.uc());
	  myTxn.commit();
  }
  else {
	  modify.addInsertAfterStep(env.db().prepare(args[1], env.context()), o, args[3], args[4]);
	  modifications = modify.execute(*env.results(), env.context(), env.uc());
  }

  if(env.verbose()) cout << modifications << " modifications made." << endl;
}

