//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: ListIndexesCommand.cpp,v 1.4 2004/11/08 16:08:08 jsnelson Exp $
//

#include "ListIndexesCommand.hpp"

using namespace DbXml;
using namespace std;

string ListIndexesCommand::getCommandName() const
{
  return "listindexes";
}

string ListIndexesCommand::getBriefHelp() const
{
  return "List the indexes of the default container";
}

string ListIndexesCommand::getMoreHelp() const
{
  return
    string("Takes no arguments. The command uses the\n") +
    string("XmlContainer::getIndexSpecification() method and iterates over the\n") +
    string("XmlIndexSpecification using the next() method.");
}

void ListIndexesCommand::execute(Args &args, Environment &env)
{
  if(args.size() != 1) {
    throw CommandException("Wrong number of arguments");
  }
  env.testContainer();

  //Retrieve the XmlIndexSpecification from the container
  XmlIndexSpecification idxSpec;
  if(env.txn()) {
	  idxSpec = env.container()->getIndexSpecification(*env.txn());
  }
  else {
	  idxSpec = env.container()->getIndexSpecification();
  }
    
  //Lets see what indexes exist on this container
  string def = idxSpec.getDefaultIndex();
  if(def != "") {
	  cout << "Default Index: " << def << endl;
  }

  string uri, name, index;
  int count = 0;
  while(idxSpec.next(uri, name, index)) {
    // Obtain the value as a string and print it to the console
    cout << "Index: " << index << " for node {" << uri << "}:" << name << endl;
    ++count;
  }

  cout << count << " indexes found." << endl;
}

