//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: QueryCommand.cpp,v 1.5 2004/10/18 22:52:13 gmf Exp $
//

#include "QueryCommand.hpp"

using namespace DbXml;
using namespace std;

string QueryCommand::getCommandName() const
{
	return "query";
}

string QueryCommand::getBriefHelp() const
{
	return "Execute an expression in the context of the entire database";
}

string QueryCommand::getMoreHelp() const
{
	return
		string("Takes one argument - the query to be run. This is run using the\n") +
		string("XmlManager::query() method, and the results are returned.");
	;
}

void QueryCommand::execute(Args &args, Environment &env)
{
	if(args.size() != 2) {
		throw CommandException("Wrong number of arguments");
	}
	env.deleteResults();

	if(env.txn()) {
		env.results() = new XmlResults(env.db().query(*env.txn(), args[1], env.context(), DBXML_LAZY_DOCS));
	}
	else {
		env.results() = new XmlResults(env.db().query(args[1], env.context(), DBXML_LAZY_DOCS));
	}

	if(env.context().getEvaluationType() == XmlQueryContext::Eager) {
		if(env.verbose()) cout << env.results()->size() << " objects returned for eager expression '"
				       << args[1] << "'" << endl << endl;
	}
	else {
		if(env.verbose()) cout << "Lazy expression '" << args[1] << "' completed" << endl << endl;
	}
}

