//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: ReturnTypeCommand.cpp,v 1.4 2004/11/11 17:13:00 gmf Exp $
//

#include "ReturnTypeCommand.hpp"

using namespace DbXml;
using namespace std;

string ReturnTypeCommand::getCommandName() const
{
	return "returntype";
}

string ReturnTypeCommand::getBriefHelp() const
{
	return "Sets the return type on the default context";
}

string ReturnTypeCommand::getMoreHelp() const
{
	return
		string("Takes the return type as an argument. Valid return types are:\n") +
		string("dv (dead values) or lv (live values).\n") +
		string("This command uses the XmlQueryContext::setReturnType() method.");
}

void ReturnTypeCommand::execute(Args &args, Environment &env)
{
	if(args.size() != 2) {
		throw CommandException("Wrong number of arguments");
	}

	string type;
	XmlQueryContext::ReturnType t = (XmlQueryContext::ReturnType)-1;

	if(args[1] == "dv") {
		t = XmlQueryContext::DeadValues;
		type = "DeadValues";
	}
	else if(args[1] == "lv") {
		t = XmlQueryContext::LiveValues;
		type = "LiveValues";
	}

	if(t == (XmlQueryContext::ReturnType)-1) {
		throw CommandException("Unknown return type: " + args[1]);
	}

	if(env.verbose()) cout << "Setting return type to: " << type << endl;
	env.context().setReturnType(t);
}

