//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002-2004
//	Sleepycat Software.  All rights reserved.
//
// $Id: Shell.cpp,v 1.2 2004/08/19 16:52:27 gareth Exp $
//

#include "Shell.hpp"
#include "ShellCommand.hpp"
#include "QuitCommand.hpp"
#include "HelpCommand.hpp"
#include "CommentCommand.hpp"
#include "IncludeCommand.hpp"

using namespace DbXml;
using namespace std;

Shell::Shell()
{
  addCommand(new QuitCommand());
  addCommand(new HelpCommand(commands_));
  addCommand(new CommentCommand);
  addCommand(new IncludeCommand);
}

Shell::~Shell()
{
  for(vector<ShellCommand*>::iterator i = commands_.begin();
      i != commands_.end(); ++i) {
    delete *i;
  }
}

bool Shell::mainLoop(std::istream &in, Environment &env) const
{
  InputParser iParser;
  ShellCommand::Args args;

  const Shell *oldShell = env.shell();
  env.shell() = this;

  bool success = true;
  do {
    if(env.interactive()) cout << endl << "dbxmlsh> ";
    env.lineNo() += iParser.parse(in, args);
    string command = args.empty() ? string("") : args.front();
    try {
      vector<ShellCommand*>::const_iterator end = commands_.end();
      vector<ShellCommand*>::const_iterator i = commands_.begin();
      for(; i != end; ++i) {
        if((*i)->matches(command)) {
          command = (*i)->getCommandName();
          (*i)->execute(args, env);
          break;
        }
      }
          
      if(i == end && !args.empty()) {
        throw CommandException("Unknown command");
      }
    }
    //catches XmlException
    catch(exception &e) {
      cerr << env.streamName() << ":" << env.lineNo() << ": " << command << " failed, ";
      cerr << e.what() << endl;
      success = false;
      if(!env.interactive()) env.quit() = true;
    }
    catch(...) {
      cerr << env.streamName() << ":" << env.lineNo() << ": " << command << " failed" << endl;
      success = false;
      if(!env.interactive()) env.quit() = true;
    }
  } while(!env.quit() && !in.eof());

  env.shell() = oldShell;
  return success;
}

void Shell::addCommand(ShellCommand *command)
{
  commands_.push_back(command);
}
